<?php namespace Builder\Users;

use Cartalyst\Sentry;
use Silex\Application;
use Symfony\Component\HttpFoundation\Cookie;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

class UsersController {

	/**
	 * Request instance.
	 * 
	 * @var Symfony\Component\HttpFoundation\Request
	 */
	private $request;

	/**
	 * Paramater bag instance.
	 * 
	 * @var Symfony\Component\HttpFoundation\ParameterBag
	 */
	private $input;

	private $app;

	/**
	 * Create new ProjectsController instance.
	 * 
	 * @param Application $app    
	 * @param Request     $request
	 */
	public function __construct($app, Request $request, $sentry, $validator)
	{	
		$this->app = $app;
		$this->sentry = $sentry;
		$this->request = $request;
		$this->validator = $validator;
		$this->input = $request->request;
	}

	public function index()
	{
		if ($this->sentry->getUser()->hasAccess('users.update') || $this->app['is_demo']) {
			return UserModel::all();
		}

		return $this->app['translator']->trans('noPermissionsGeneric');
	}

	public function delete($id)
	{
		$u = $this->sentry->getUser();

		if ($u->hasAccess('users.delete') && $u->id != $id) {
			return UserModel::destroy($id);
		}

		return new Response($this->app['translator']->trans('noPermissionsGeneric'), 403);
	}

	public function login()
	{
		$creds = $this->input->all();

		//do some basic validation just in case
		if ($this->validator->fails($creds, 'login')) {
			return new Response(json_encode($this->validator->errors), 400);
		}

		$error = array();

		try {
			$user = $this->sentry->authenticate($creds, true);
		}
		catch (\Cartalyst\Sentry\Users\WrongPasswordException $e) {
		    $error['*'] = $this->app['translator']->trans('wrongEmailOrPassword');
		}
		catch (\Cartalyst\Sentry\Users\UserNotFoundException $e) {
		    $error['*'] = $this->app['translator']->trans('wrongEmailOrPassword');
		}
		catch (\Cartalyst\Sentry\Throttling\UserSuspendedException $e) {
		    $error['*'] = $this->app['translator']->trans('userSuspended');
		}
		catch (\Cartalyst\Sentry\Throttling\UserBannedException $e) {
		    $error['*'] = $this->app['translator']->trans('userBanned');
		}

		if (count($error)) {
			return new Response(json_encode($error), 400);
		}

		$response = new Response($user, 200);
		$response->headers->setCookie(new Cookie('blUser', $user, 0, '/', null, false, false));

		return $response;
	}

	/**
	 * Register a new user.
	 * 
	 * @return Response
	 */
	public function store()
	{
		$settings  = $this->app['settings'];
        $creds     = $this->input->all();
        $dontLogin = $this->input->get('create');

		if ( ! $settings['enable_registration'] && (! isset($creds['create']) || ! $creds['create'])) {
			return new Response($this->app['translator']->trans('registrationDisabled'), 403);
		}

		if ( ! isset($creds['create'])) {
			$creds['create'] = false;
		}
		
		if ($this->validator->fails($creds, 'register')) {
			return new Response(json_encode($this->validator->errors), 400);
		}

		//god damn symfony
		$this->input->remove('create');
		unset($creds['create']);

		try {
		    $this->input->remove('repeatPassword');

		    $input = $this->input->all();
		    $input['permissions'] = json_decode($this->app['settings']['permissions'], true);
		    
			$user = $this->sentry->register($input, true);

			if ( ! $dontLogin) {
				$user = $this->sentry->authenticate($this->input->all(), true);
			}
			
		} catch (\Cartalyst\Sentry\Users\UserExistsException $e) {
		    return new Response(json_encode(array('email' => $this->app['translator']->trans('emailTaken'))), 400);
		}

		$response = new Response($user, 201);
		
		if ( ! $dontLogin) {
			$response->headers->setCookie(new Cookie('blUser', $user, 0, '/', null, false, false));
		}

		return $response;
	}

	public function modifyPermissions($id)
	{
		if ($this->app['is_demo']) {
			return new Response('Permission modifications are disabled on demo site.', 403);
		}

		if ($this->sentry->getUser()->hasAccess('users.update')) {
			$user = UserModel::find($id);

			//only superusers can modify other superusers permissions
			if ( ! $user || ($user->isSuperUser() && ! $this->sentry->getUser()->isSuperUser())) {
				return $this->app['translator']->trans('noPermissionsGeneric');
			}

			$user->update(array('permissions' => $this->input->get('permissions')));

			return new Response('Permissions modified successfully.', 201);
		}

		return $this->app['translator']->trans('noPermissionsGeneric');
	}

	public function assignPermissionsToAll()
	{
		if ( ! $this->sentry->getUser()->hasAccess('superuser') || ! $this->input->has('permissions')) {
			return new Response($this->app['translator']->trans('noPermissionsGeneric'), 403);
		}

		UserModel::whereNull('permissions')->update(array('permissions' => $this->input->get('permissions')));

		return new Response($this->app['translator']->trans('permissionsUpdated'), 200);
	}

	public function logout()
	{
		$this->sentry->logout();

		return new Response('', 200);
	}
}